# -*- coding: utf-8 -*-
import copy
from django.utils import simplejson
from django.utils.text import capfirst
from forms import MultiFileField as MultiFileUploadField, ClearableMultiFileInput
from django.db.models.fields.files import FileField, FieldFile
from django.core.files.base import File
from django.core.validators import EMPTY_VALUES
from django.utils.translation import ugettext_lazy as _


class MultiFileDescriptor(object):
    """
    The descriptor for the file attribute on the model instance. Returns a
    FieldFile when accessed so you can do stuff like::

        >>> instance.file.size

    Assigns a file object on assignment so you can do::

        >>> instance.file = File(...)

    """
    def __init__(self, field):
        self.field = field

    def __get__(self, instance=None, owner=None):
        if instance is None:
            raise AttributeError(
                "The '%s' attribute can only be accessed from %s instances."
                % (self.field.name, owner.__name__))

        # This is slightly complicated, so worth an explanation.
        # instance.file`needs to ultimately return some instance of `File`,
        # probably a subclass. Additionally, this returned object needs to have
        # the FieldFile API so that users can easily do things like
        # instance.file.path and have that delegated to the file storage engine.
        # Easy enough if we're strict about assignment in __set__, but if you
        # peek below you can see that we're not. So depending on the current
        # value of the field we have to dynamically construct some sort of
        # "thing" to return.

        # The instance dict contains whatever was originally assigned
        # in __set__.
        files = instance.__dict__[self.field.name]
        try:
            files = simplejson.loads(files)
        except:
            pass

        files_instance = []
        # If this value is a string (instance.file = "path/to/file") or None
        # then we simply wrap it with the appropriate attribute class according
        # to the file field. [This is FieldFile for FileFields and
        # ImageFieldFile for ImageFields; it's also conceivable that user
        # subclasses might also want to subclass the attribute class]. This
        # object understands how to convert a path to a file, and also how to
        # handle None.
        if files not in EMPTY_VALUES:
            for file in files:
                if isinstance(file, basestring):
                    attr = self.field.attr_class(instance, self.field, file)
                    files_instance.append(attr)

            # Other types of files may be assigned as well, but they need to have
            # the FieldFile interface added to the. Thus, we wrap any other type of
            # File inside a FieldFile (well, the field's attr_class, which is
            # usually FieldFile).
                elif isinstance(file, File) and not isinstance(file, FieldFile):
                    file_copy = self.field.attr_class(instance, self.field, file.name)
                    file_copy.file = file
                    file_copy._committed = False
                    files_instance.append(file_copy)

                # Finally, because of the (some would say boneheaded) way pickle works,
                # the underlying FieldFile might not actually itself have an associated
                # file. So we need to reset the details of the FieldFile in those cases.
                elif isinstance(file, FieldFile) and not hasattr(file, 'field'):
                    file.instance = instance
                    file.field = self.field
                    file.storage = self.field.storage
                else:
                    files_instance.append(file)
        # That was fun, wasn't it?
        return files_instance

    def __set__(self, instance, value):
        instance.__dict__[self.field.name] = value


class MultiFileField(FileField):
    # The class to wrap instance attributes in. Accessing the file object off
    # the instance will always return an instance of attr_class.
    attr_class = FieldFile

    # The descriptor to use for accessing the attribute off of the class.
    descriptor_class = MultiFileDescriptor

    description = _("Files")

    # Internal type for creation database (logtext)
    def get_internal_type(self):
        return "TextField"

    def get_prep_value(self, value):
        "Returns field's value prepared for saving into a database."
        # Need to convert File objects provided via a form to unicode for database insertion
        if value in EMPTY_VALUES:
            return None
        unicode_values = []
        for file in value:
            unicode_values.append(unicode(file))
        return simplejson.dumps(unicode_values)

    def pre_save(self, model_instance, add):
        "Returns field's value just before saving."
        clear = model_instance.__dict__.get('_clear')
        file_list = model_instance.__dict__[self.name]
        if file_list:
            for file in file_list:
                if file and not file._committed:
                    # Commit the file to storage prior to saving the model
                    file.save(file.name, file, save=False)
        if clear:
            fs = copy.copy(file_list)
            for file in clear:
                id = int(file.split('-')[1])
                fs[id].delete(save=False)
        file_list = filter(lambda x: x, file_list)
        return file_list

    def save_form_data(self, instance, data):
        # Important: None means "no change", other false value means "clear"
        # This subtle distinction (rather than a more explicit marker) is
        # needed because we need to consume values that are also sane for a
        # regular (non Model-) Form to find in its cleaned_data dictionary.
        if data is not None:
            # This value will be converted to unicode and stored in the
            # database, so leaving False as-is is not acceptable.
            if not data:
                data = ''
            # getting upload/delete data
            try:
                clear = data.get('clear')
                files = data.get('upload')
            # Exception means that field wasn't changed
            except AttributeError:
                files = data
                clear = None
            if clear:
                setattr(instance, '_clear', clear)
            if files:
                setattr(instance, self.name, files)

    def formfield(self, **kwargs):
        """
        Returns a django.forms.Field instance for this database Field.
        """
        defaults = {'required': not self.blank,
                    'label': capfirst(self.verbose_name),
                    'help_text': self.help_text,
                   }
        if self.has_default():
            if callable(self.default):
                defaults['initial'] = self.default
                defaults['show_hidden_initial'] = True
            else:
                defaults['initial'] = self.get_default()
        defaults.update(kwargs)
        defaults['widget'] = ClearableMultiFileInput
        return MultiFileUploadField(**defaults)

    def south_field_triple(self):
        "Returns a suitable description of this field for South."
        # We'll just introspect the _actual_ field.
        from south.modelsinspector import introspector
        field_class = "django.db.models.fields.TextField"
        args, kwargs = introspector(self)
        # That's our definition!
        return (field_class, args, kwargs)



# -*- coding: utf-8 -*-
from django.forms.fields import FileField
from django.forms.widgets import CheckboxInput, FILE_INPUT_CONTRADICTION
from django.forms.widgets import Input
from django.utils.translation import ugettext, ugettext_lazy as _
from django.utils.html import escape, conditional_escape
from django.utils.encoding import force_unicode
from django.utils.safestring import mark_safe
from django.utils.datastructures import MultiValueDict, MergeDict
from django.core.exceptions import ValidationError
# Provide this import for backwards compatibility.
from django.core.validators import EMPTY_VALUES
import re

FILE_INPUT_EMPTY_VALUE = object()


class MultiFileInput(Input):
    input_type = 'file'
    needs_multipart_form = True

    def render(self, name, value, attrs=None):
        if attrs is None:
            attrs = {}

        name += '[]'
        attrs['multiple'] = 'multiple'

        return super(MultiFileInput, self).render(name, None, attrs=attrs)

    def value_from_datadict(self, data, files, name):
        """
        File widgets take data from FILES, not POST
        we need to add [] because it's from w3c recommendation
        """
        name += '[]'
        if isinstance(files, (MultiValueDict, MergeDict)):
            return files.getlist(name)
        return files.get(name, None)

    def _has_changed(self, initial, data):
        print data
        if data is None:
            return False
        return True


class ClearableMultiFileInput(MultiFileInput):
    initial_text = ugettext('Currently')
    input_text = ugettext('Change')
    clear_checkbox_label = ugettext('Delete')

    template_with_initial = '<p class="file-upload">%(initial_text)s: %(initial)s %(clear_template)s</p>'

    template_with_clear = '<span class="clearable-file-input">%(clear)s <label for="%(clear_checkbox_id)s">%(clear_checkbox_label)s</label></span>'

    def clear_checkbox_name(self, i, name):
        """
        Given the name of the file input, return the name of the clear checkbox
        input.
        """
        return name + '-' + str(i) + '-clear'

    def clear_checkbox_id(self, name):
        """
        Given the name of the clear checkbox input, return the HTML id for it.
        """
        return name + '_id'

    def render(self, name, value, attrs=None):
        substitutions = {
            'initial_text': self.initial_text,
            'input_text': self.input_text,
            'clear_template': '',
            'clear_checkbox_label': self.clear_checkbox_label,
        }
        input_template = '%(input)s'
        init_template = ''
        substitutions['input'] = super(ClearableMultiFileInput, self).render(name, value, attrs)
        if value is not None:
            # for validation empty value
            for i, file in enumerate(value):
                if file and hasattr(file, "url"):
                    template = self.template_with_initial
                    substitutions['initial'] = ('<a href="%s">%s</a>&nbsp;'
                                                           % (escape(file.url),
                                                              escape(force_unicode(file))))
                    if self.is_required or len(value) > 1:
                        checkbox_name = self.clear_checkbox_name(i, name)
                        checkbox_id = self.clear_checkbox_id(checkbox_name)
                        substitutions['clear_checkbox_name'] = conditional_escape(checkbox_name)
                        substitutions['clear_checkbox_id'] = conditional_escape(checkbox_id)
                        substitutions['clear'] = CheckboxInput().render(checkbox_name, False, attrs={'id': checkbox_id})
                        substitutions['clear_template'] = self.template_with_clear % substitutions
                init_template += (template % substitutions)
            init_template += '<input type="hidden" value="%s" name="total_files">' % len(value)
        return mark_safe(init_template + (input_template % substitutions))

    def value_from_datadict(self, data, files, name):
        upload = super(ClearableMultiFileInput, self).value_from_datadict(data, files, name)
        clear = self._get_clear_list(data, files, name)
        total_files = data.get('total_files', '1')
        if total_files.isdigit():
            total_files = int(total_files)
        if clear not in EMPTY_VALUES:
            if upload:
                # If the user contradicts themselves (uploads a new file AND
                # checks the "clear" checkbox), we return a unique marker
                # object that FileField will turn into a ValidationError.
                return FILE_INPUT_CONTRADICTION
                # If the user delete all files and field is not allow empty values,
                # we return a unique marker
                # object that FileField will turn into a ValidationError.
            if self.is_required and total_files <= len(clear):
                return FILE_INPUT_EMPTY_VALUE
            return {'clear': clear}
        if upload not in EMPTY_VALUES:
            return {'upload': upload}
        return None


    def _get_clear_list(self, data, files, name):
        pattern = re.compile('%s-\d+-clear' % name)
        clear = []
        for key, value in data.iteritems():
            if pattern.match(key) and value == 'on':
                clear.append(key)
        return clear




class MultiFileField(FileField):
    widget = ClearableMultiFileInput
    default_error_messages = {
        'empty_multiply': _("The submitted field is empty! If you want to change all your files, please, select files!")
    }

    def to_python(self, data):
        if data in EMPTY_VALUES:
            return None
        if data is FILE_INPUT_EMPTY_VALUE:
            raise ValidationError(self.error_messages['empty_multiply'])

        # UploadedFile objects should have name and size attributes.
        for d in data.get('upload', []):
            try:
                file_name = d.name
                file_size = d.size
            except AttributeError:
                raise ValidationError(self.error_messages['invalid'])

            if self.max_length is not None and len(file_name) > self.max_length:
                error_values = {'max': self.max_length, 'length': len(file_name)}
                raise ValidationError(self.error_messages['max_length'] % error_values)
            if not file_name:
                raise ValidationError(self.error_messages['invalid'])
            if not self.allow_empty_file and not file_size:
                raise ValidationError(self.error_messages['empty'])
        return data


    def bound_data(self, data, initial):
        if data in (None, FILE_INPUT_CONTRADICTION, FILE_INPUT_EMPTY_VALUE):
            return initial
        return data


class AdminMultiFileInputWidget(ClearableMultiFileInput):
    template_with_initial = (u'<p class="file-upload">%s</p>'
                            % ClearableMultiFileInput.template_with_initial)
    template_with_clear = (u'<span class="clearable-file-input">%s</span>'
                           % ClearableMultiFileInput.template_with_clear)



#!/usr/bin/env python
from distutils.core import setup

setup(
    name='django-multifilefield',
    version='1.0',
    description='Django model and form fields for storring html5 forms multiupload data',
    author='Sergey Maltsev',
    author_email='alternativshik@gmail.com',
    url='https://github.com/Cyberic/django-multifilefield',
    packages=[
        'multifilefield',
    ]
)
